#!usr/central/bin/perl



# iTRAQ Ratio Calculator: i-Tracker

# Version 1.2-Centroided



# Created by Ian Shadforth with assistance from Tom Dunkley.



# This program takes as input a folder of mgf or dta files containing UNCENTROIDED peaks (m/z) and

# ion count as produced by programs such as Mascot distiller, wiff2dta etc. These are used to

# calculate the areas of the reporter ion peaks, using a peak width of 0.1Da -

# from n.05 to n.15, and using the trapeziod approximation of the area under

# the peak.



# User Instructions:

# Type at the command prompt...

# perl i-Tracker.pl

# Then hit enter.

# Enter the directory containing .mgf or .dta files when prompted.

# Enter reporter ion count threshold when prompted.

# Enter purity correction factors (if needed) when prompted.

# Results will appear in the directory containing the Filename.mgf or .dta files as Filename.itq



# The user will be prompted for a threshold below which

# reporter ion information should be ignored - this threshold is in ion-counts

# per time (as set by the MS).

# The purity correction factors, supplied by ABI

# with the iTRAQ reagents, may be supplied as 16 comma-separated values. If

# purity correction is not desired do not enter any values (or enter 16 comma-

# separated zeroes - the effect will be the same).



# Two outputs are produced:



# Output 1 (.hum for human readability) consists of a flat text file (CSV) with the following format:

# >Title (file processed)

# >When file processed

# Block heading (eg "m/zprecursor,charge,elution time" to be used when matching spectal IDs to quantitative info)

# Mascot title info (if present)

# Areas,114, 115, 116, 117,, Ratios, 114, 115, 116, 117,,Errors,114,115,116,117    Peak area normalised and given to 3 d.p.

# Norm, area,area,area,area,,114,      r,   r,   r,   r,,114,   err,err,err,err    where r is the ratio of peaks, top/left

# UT?,  UT/a,UT/a,UT/a,UT/a,,115,      r,   r,   r,   r,,115,   err,err,err,err    Ratios given to 3 d.p. or UT (under threshold) or NA

# ,,,,,,                     116,      r,   r,   r,   r,,116,   err,err,err,err    Error as +/- %age OR NA

# ,,,,,,                     117,      r,   r,   r,   r,,117,   err,err,err,err

# >

# Next block...



# This format opens well in Excel for human-readable viewing.



# Output 2 (.mcn for machine readability) is simpler in structure for easier processing:

# Title (file processed), When file processed

# Block heading, Mascot title, 4 areas, 4 UT/areas, 16 ratios, 16 errors          ### The order of these is as if

# Data from 1st spectrum                                                          ### the human readable file was

# Data from 2nd spectrum...                                                       ### extended block by block, so row 2 follows row 1 in each section



# INTERNAL NOTES:

# i-Tracker.pl is based on iTRAQ_R_v5b.pl

# Version 1.1 adds second (.mcn) output type to version 1

# Version 1.2 exposes the integration boundaries to manual adjustment as per

# BMC Genomics review.

# Centroided version takes centroided input peak lists. NB It is the users'

# responsibility to ensure that the centroided intensities are directly related

# to what would have been the peak areas.

# Peak detection range fixed at +/- 0.5Da.

# Errors in this version are meaningless and so are not reported.





use Time::localtime;

use Math::MatrixReal;



my @localtime = @{localtime(time)};

if (length($localtime[2]) == 1) {

    $localtime[2] = join("",("0",$localtime[2]));

}

if (length($localtime[1]) == 1) {

    $localtime[1] = join("",("0",$localtime[1]));

}

my $timeprocessed = join(":",($localtime[2],$localtime[1]));

my $dateprocessed = join("/",($localtime[3],($localtime[4] + 1),($localtime[5] + 1900)));



#use the command line arguments



my $directory = $ARGV[0];

chomp $directory;

if ($directory eq "") {

    die "Invalid directory - please re-run program.\n";

}



#print "Enter peak integration range, order -,+ (eg: 0.05,0.05).\nOR leave blank for default of -/+ 0.05 Da from nominal reporter ion mass.\n";

#my $int_range = <STDIN>;

#chomp $int_range;

#if ($int_range eq "") {

    $int_range = "0.5,0.5";

#}

my @int_range = split(",",$int_range);

#if (($int_range[0] + $int_range[1]) > 1) {

#    die "The input peak integration ranges cross over between peaks (sum is > 1).\nPlease re-run program.\n";

#}



#print "Enter reporter ion intensity threshold:\n";

my $threshold = $ARGV[1];

chomp $threshold;

if ($threshold eq "") {

    die "Invalid threshold - please re-run program.\n";

}



#print "Enter 16 comma-separated purity-correction percentages in the order: -2, -1, +1, +2 for each consecutive tag\n";

#the file name for the correction factors

my $filename = $ARGV[2];

my $purecorrect;

open(FILE,"<$filename") or print "WARNING: (itracker)unable to open the correction file\n";

 while(my $line = <FILE>)

 {

 $purecorrect = $line;

 }

close FILE;



chomp $purecorrect;

my $detA = 0;

if ($purecorrect eq "0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0") {

    print "WARNING: (itracker) Proceeding without purity correction.\n";

}

elsif ($purecorrect eq "") {

    print "WARNING: (itracker)Proceeding without purity correction.\n";

}

else {

    $detA = &determine($purecorrect);

}

    

#opendir(DIR,$directory) || die "\nThe directory $directory is unavailable.\n";

#my @dtafiles = readdir(DIR);

$dtafiles[0] = $directory;

$directory = "";

my $dtaflag = 0;



### REWRITE so it reads from the file....



foreach $file (@dtafiles) {

    if ($file !~ /\.mgf$/ && $file !~ /\.dta$/) {

        if ($file eq ".") {}

        elsif ($file eq "..") {}

        elsif ($file =~ /\.itq$/) {}

        else {

            #print "WARNING: The file $file does not terminate in .mgf or .dta\nThis will not have been processed.\n";

        }

    }

    elsif ($file =~ /\.mgf$/) {

        #sysopen (INPUT,"$directory/$file",O_RDONLY) || die "WARNING: ITRAQ_R cannot open $file!\n";

	sysopen (INPUT,"$file",O_RDONLY) || die "WARNING: ITRAQ_R cannot open $file!\n";

        #print "Currently processing $file.\n";



        my $outend = $file;

        $outend =~ s/\.mgf/\itq.hum/;

        

	#my $outname = join("/",($directory,$outend));

        my $outname = $outend;

	open (OUTPUT,">$outname") || die "WARNING: ITRAQ_R cannot create $outname!\n";

        print OUTPUT ">ITRAQ_R output for $directory/$file\n>Processed on $dateprocessed at $timeprocessed\n";

        

        my $macoutname = $outname;

        $macoutname =~ s/\.hum/\.mcn/;

        open (OUTPUT2,">$macoutname") || die "WARNING: ITRAQ_R cannot create $macoutname!\n";

        print OUTPUT2 "ITRAQ_R output for $directory/$file,Processed on $dateprocessed at $timeprocessed\n";

        print OUTPUT2 "Block heading, Mascot Title (if any), 4 areas (csv),,,, 4 UT/areas (csv),,,, 16 ratios (csv (114 115 116 117)/114,,,, (114 115 116 117)/115,,,, etc...),,,,,,,,16 errors (csv as ratios)\n";

        

        @spectrumdata = ();

        while (<INPUT>) {

            $line = $_;

            if ($line =~ /^END IONS/) {

                push(@spectrumdata,$line);

                &spectrumproc(\@spectrumdata,"mgf","NA");

                undef @spectrumdata;

                @spectrumdata = ();

            }

            elsif ($line =~ /^SEARCH/) {}

            elsif ($line =~ /^REPTYPE/) {}

            elsif ($line =~ /^\s/) {}

            else {

                push(@spectrumdata,$line);

            }

        }

        #print "ITRAQ_R completed processing $file.\nTo view the results open $outname.\n";

        close OUTPUT;

        close OUTPUT2;

        close INPUT;

    }

    else {

        #open (INPUT,"$directory/$file") || die "WARNING: ITRAQ_R cannot open $file!\n";

        open (INPUT,"$file") || die "WARNING: ITRAQ_R cannot open $file!\n";

	my @spectrumdata = <INPUT>;

        close INPUT;





        if ($dtaflag == 0) {

            #print "Currently processing $directory\n";

            my $outend = "i-Tracker.hum";



            my $outname = join("/",($directory,$outend));

            open (OUTPUT,">$outname") || die "WARNING: ITRAQ_R cannot create $outname!\n";

            print OUTPUT ">ITRAQ_R output for $directory/$file\n>Processed on $dateprocessed at $timeprocessed\n";



            my $macoutname = $outname;

            $macoutname =~ s/\.hum/\.mcn/;

            open (OUTPUT2,">$macoutname") || die "WARNING: ITRAQ_R cannot create $macoutname!\n";

            print OUTPUT2 "ITRAQ_R output for $directory/$file,Processed on $dateprocessed at $timeprocessed\n";

            print OUTPUT2 "Block heading, Mascot Title (if any), 4 areas (csv),,,, 4 UT/areas (csv),,,, 16 ratios (csv (114 115 116 117)/114,,,, (114 115 116 117)/115,,,, etc...),,,,,,,,16 errors (csv as ratios)\n";



            $dtaflag = 1;

        }



        &spectrumproc(\@spectrumdata,"dta",$file);

        undef @spectrumdata;

        



        

    }

}



if ($dtaflag == 1) {

    #print "To view the results open ${directory}/i-Tracker.hum and i-Tracker.mcn.\n";

    close OUTPUT;

    close OUTPUT2;

}

closedir DIR;

#print "\aITRAQ_R completed processing $directory.\n";

exit;



sub spectrumproc {

  my @spectrum = @{$_[0]};

  my $type = $_[1];

  my $filename = $_[2]; # NA if mgf

  if ($type eq "mgf") {

      if ($spectrum[0] !~ /^BEGIN IONS/) {

            print "WARNING: (itracker) Incorrect input format for mgf file!\n";

            die;

      }

      else {

          shift(@spectrum); #remove BEGIN IONS

          my $pepmass = shift(@spectrum);

          my $charge = shift(@spectrum);

          my $title = shift(@spectrum);

          pop(@spectrum); #remove END IONS



          $pepmass =~ s/PEPMASS=//;

          $charge =~ s/CHARGE=//;

          $charge =~ s/\r//;

          chomp $charge;

    

          my @titleinfo = split(",",$title);

          my $elution = $titleinfo[2];

          $elution =~ s/ Elution: //;



          $pepmass = &round($pepmass * 100);

          $pepmass = ($pepmass / 100);

        

          print OUTPUT "MASCOT $pepmass $charge $elution\n";

          print OUTPUT "$title";

          

          $title =~ s/[\r\n]//g;

          print OUTPUT2 "MASCOT $pepmass $charge $elution,$title,";

          

      }

  }

  elsif ($type eq "dta") {

      my $parentinfo = shift(@spectrum);

      my @parentinfo = split(/\s/,$parentinfo);



      my @fileinfo = split(/\./,$filename);



      my $pepmass = $parentinfo[0];

      my $charge = $parentinfo[1];



      $pepmass = &round($pepmass * 100);

      $pepmass = ($pepmass / 100);

      

      print OUTPUT "DTA $pepmass $charge\n";

      print OUTPUT "$filename\n";

      

      print OUTPUT2 "DTA $pepmass $charge,$filename,";

  }



    # Get reporter ion peaks

    

    my %i114 = ();

    my %i115 = ();

    my %i116 = ();

    my %i117 = ();

    

    

    

    

    foreach $peak (@spectrum) {

        my @massint = split(/\s/,$peak);

        my $mass = $massint[0];

        my $int = $massint[1];

        if ($mass >= (114.1 - $int_range[0])) {

            if ($mass <= (117.1 + $int_range[1])) {

                if ($mass >= (117.1 - $int_range[0])) {

                    $i117{$mass} = $int;

                }

                elsif ($mass <= (116.1 + $int_range[1])) {

                    if ($mass >= (116.1 - $int_range[0])) {

                        $i116{$mass} = $int;

                    }

                    elsif ($mass <= (115.1 + $int_range[1])) {

                        if ($mass >= (115.1 - $int_range[0])) {

                            $i115{$mass} = $int;

                        }

                        elsif ($mass <= (114.1 + $int_range[1])) {

                            $i114{$mass} = $int;

                        }

                    }

                }

            }

        }

    }

    

    # Get max int for each reporter ion

    

    #my $iT1 = 0;     # 114.1 area

    #my $iT2 = 0;     # 115.1

    #my $iT3 = 0;     # 116.1

    #my $iT4 = 0;     # 117.1

    

    my $iT1max = 0;     # 114.1 maximum peak intensity

    my $iT2max = 0;     # 115.1

    my $iT3max = 0;     # 116.1

    my $iT4max = 0;     # 117.1

    

    my $thresholdflag = 0;

    ($iT1max) = &peakarea(\%i114);

    ($iT2max) = &peakarea(\%i115);

    ($iT3max) = &peakarea(\%i116);

    ($iT4max) = &peakarea(\%i117);

    



    

    # Correct areas using purity correction figures...

    # This uses the Cramer determinant method for solving the

    # simultaneous equations.



    my $pure1 = 0;

    my $pure2 = 0;

    my $pure3 = 0;

    my $pure4 = 0;

    

    if ($detA == 0) {

        $pure1 = $iT1max;

        $pure2 = $iT2max;

        $pure3 = $iT3max;

        $pure4 = $iT4max;

    }

    else {

        ($pure1,$pure2,$pure3,$pure4) = &purecorr($iT1max,$iT2max,$iT3max,$iT4max,$purecorrect);

    }

    

    # Normalise intensities...

    my ($puren1,$puren2,$puren3,$puren4) = &areanorm($pure1,$pure2,$pure3,$pure4);



    # Area or UT?...  NB For this version, read "area" as intensity.

    my $areaUT1 = &aUT($iT1max,$puren1);

    my $areaUT2 = &aUT($iT2max,$puren2);

    my $areaUT3 = &aUT($iT3max,$puren3);

    my $areaUT4 = &aUT($iT4max,$puren4);



    # Calculate ratios



    my $r21 = 0;

    my $r31 = 0;

    my $r41 = 0;

    my $r32 = 0;

    my $r42 = 0;

    my $r43 = 0;

    

    my $r21err = 0;

    my $r31err = 0;

    my $r41err = 0;

    my $r32err = 0;

    my $r42err = 0;

    my $r43err = 0;

    

    

    ($r21,$r21err) = &ratio($puren2,$puren1,$iT2max,$iT1max);

    ($r31,$r31err) = &ratio($puren3,$puren1,$iT3max,$iT1max);

    ($r41,$r41err) = &ratio($puren4,$puren1,$iT4max,$iT1max);

    ($r32,$r32err) = &ratio($puren3,$puren2,$iT3max,$iT2max);

    ($r42,$r42err) = &ratio($puren4,$puren2,$iT4max,$iT2max);

    ($r43,$r43err) = &ratio($puren4,$puren3,$iT4max,$iT3max);

    

    if ($r21 =~ /^\d/) {

        $r21 = &threedp($r21);

    }

    if ($r31 =~ /^\d/) {

        $r31 = &threedp($r31);

    }

    if ($r41 =~ /^\d/) {

        $r41 = &threedp($r41);

    }

    if ($r32 =~ /^\d/) {

        $r32 = &threedp($r32);

    }

    if ($r42 =~ /^\d/) {

        $r42 = &threedp($r42);

    }

    if ($r43 =~ /^\d/) {

        $r43 = &threedp($r43);

    }

    

    # Calculate ratio reciprocals

    

    my $r12 = "UT";

    my $r13 = "UT";

    my $r14 = "UT";

    my $r23 = "UT";

    my $r24 = "UT";

    my $r34 = "UT";

    

    if ($r21 > 0) {

        $r12 = &threedp(1 / $r21);

    }

    if ($r31 > 0) {

        $r13 = &threedp(1 / $r31);

    }

    if ($r41 > 0) {

        $r14 = &threedp(1 / $r41);

    }

    if ($r32 > 0) {

        $r23 = &threedp(1 / $r32);

    }

    if ($r42 > 0) {

        $r24 = &threedp(1 / $r42);

    }

    if ($r43 > 0) {

        $r34 = &threedp(1 / $r43);

    }

    

    # Errors will be the same, so no need to recalc, except for indiv error components

    my $err1 = "NA";

    my $err2 = "NA";

    my $err3 = "NA";

    my $err4 = "NA";

    

    

    if ($iT1max > 0) {

        $err1 = &round((0.5 / $iT1max) * 100);

    }

    if ($iT2max > 0) {

        $err2 = &round((0.5 / $iT2max) * 100);

    }

    if ($iT3max > 0) {

        $err3 = &round((0.5 / $iT3max) * 100);

    }

    if ($iT4max > 0) {

        $err4 = &round((0.5 / $iT4max) * 100);

    }

    

    print OUTPUT "Areas,114, 115, 116, 117,, Ratios, 114, 115, 116, 117\n";

    print OUTPUT "Norm,$puren1,$puren2,$puren3,$puren4,,114,1,$r21,$r31,$r41\n";

    print OUTPUT "UT?,$areaUT1,$areaUT2,$areaUT3,$areaUT4,,115,$r12,1,$r32,$r42\n";

    print OUTPUT ",,,,,,116,$r13,$r23,1,$r43\n";

    print OUTPUT ",,,,,,117,$r14,$r24,$r34,1\n";

    print OUTPUT ">\n";

    

    print OUTPUT2 "$puren1,$puren2,$puren3,$puren4,$areaUT1,$areaUT2,$areaUT3,$areaUT4,";

    print OUTPUT2 "1,$r21,$r31,$r41,$r12,1,$r32,$r42,$r13,$r23,1,$r43,$r14,$r24,$r34,1\n";

    #print OUTPUT2 "$err1,$r21err,$r31err,$r41err,$r21err,$err2,$r32err,$r42err,$r31err,$r32err,$err3,$r43err,$r41err,$r42err,$r43err,$err4\n";





    return;

}



sub ratio {

    my $top = $_[0];

    my $bottom = $_[1];

    my $topmax = $_[2];

    my $bottommax = $_[3];

    my $ratio = 0;

    my $toterr = "NA";

    if ($topmax <= $threshold) {

        $ratio = "UT";

    }

    elsif ($bottommax <= $threshold) {

        $ratio = "UT";

    }

    elsif ($bottom == 0) {

        $ratio = "NA";

    }

    else {

        $ratio = ($top / $bottom);

        my $toperr = (0.5 / $topmax);        # Quantisation error at max

        my $bottomerr = (0.5 / $bottommax);

        $toterr = &round(($toperr + $bottomerr)* 100);

    }

    

    return ($ratio, $toterr);

}



sub peakarea {

    my %rephash = %{$_[0]};

    my @keys = keys(%rephash);

    my @sortkeys = sort {$a <=> $b} @keys;

    my $l = @sortkeys;

    my $i = 0;

    my $max = 0;

    #my $area = 0;

    if ($l == 1) {

        $max = $rephash{$sortkeys[$i]};

    }

    else {

        while ($i < ($l - 1)) {

        #my $subarea = 0;

            my $int1 = $rephash{$sortkeys[$i]};

        #my $int2 = $rephash{$sortkeys[($i + 1)]};

        #my $height = (($int1 + $int2) / 2);

        #my $width = ($sortkeys[($i + 1)] - $sortkeys[$i]);

        #$area = ($area + ($height * $width));



            if ($int1 > $max) {

                $max = $int1;

            }



            $i++;

        }

    }

    

    return ($max);

}



sub areanorm {

    my @areas = @_;



    my $totarea = ($areas[0] + $areas[1] + $areas[2] + $areas[3]);

    my $a1 = 0;

    my $a2 = 0;

    my $a3 = 0;

    my $a4 = 0;

    

    if ($totarea == 0) { }

    else {

        $a1 = &threedp($areas[0] / $totarea);

        $a2 = &threedp($areas[1] / $totarea);

        $a3 = &threedp($areas[2] / $totarea);

        $a4 = &threedp($areas[3] / $totarea);

    }

    return($a1,$a2,$a3,$a4);

}

    



sub round {

    my $floater = $_[0];

    $intpart = int($floater);

    $decpart = ($floater - $intpart);

    if ($decpart < 0.5) {}

    else {

        $intpart++;

    }

    return($intpart);

}



sub threedp {

    my $raw = $_[0];

    $raw = &round($raw * 1000);

    $raw = ($raw / 1000);

    return($raw);

}



sub determine {

    my $corrs = $_[0];

    my @purecorrs = split(/,/,$corrs);



    my ($a,$b,$c,$d,$e,$f,$g,$h,$i,$j,$k,$l,$m,$n,$o,$p) =

        ($purecorrs[0],$purecorrs[4],$purecorrs[8],$purecorrs[12],

         $purecorrs[1],$purecorrs[5],$purecorrs[9],$purecorrs[13],

         $purecorrs[2],$purecorrs[6],$purecorrs[10],$purecorrs[14],

         $purecorrs[3],$purecorrs[7],$purecorrs[11],$purecorrs[15]);



    my $w = (100 - ($a + $e + $i + $m));

    my $x = (100 - ($b + $f + $j + $n));

    my $y = (100 - ($c + $g + $k + $o));

    my $z = (100 - ($d + $h + $l + $p));



    my $A = Math::MatrixReal->new_from_rows([ [$w,$f,$c,0],

                                              [$i,$x,$g,$d],

                                              [$m,$j,$y,$h],

                                              [0,$n,$k,$z] ]);

    my $detA = $A->det();

    #print "detA is: $detA\n";

    if ($detA == 0) {

    print "WARNING: (itracker) No-unique solutions to purity correction calculations.\nWARNING! Proceeding without purity correction.\n";

    }

    

    return($detA);

}



sub purecorr {

    my ($in1,$in2,$in3,$in4,$corrs) = @_;

    my @purecorrs = split(/,/,$corrs);

    my ($a,$b,$c,$d,$e,$f,$g,$h,$i,$j,$k,$l,$m,$n,$o,$p) =

        ($purecorrs[0],$purecorrs[4],$purecorrs[8],$purecorrs[12],

         $purecorrs[1],$purecorrs[5],$purecorrs[9],$purecorrs[13],

         $purecorrs[2],$purecorrs[6],$purecorrs[10],$purecorrs[14],

         $purecorrs[3],$purecorrs[7],$purecorrs[11],$purecorrs[15]);



    my $w = (100 - ($a + $e + $i + $m));

    my $x = (100 - ($b + $f + $j + $n));

    my $y = (100 - ($c + $g + $k + $o));

    my $z = (100 - ($d + $h + $l + $p));

    

    my $one = Math::MatrixReal->new_from_cols([    [$in1,$in2,$in3,$in4],

                                                       [$f,$x,$j,$n],

                                                       [$c,$g,$y,$k],

                                                       [0,$d,$h,$z] ]);



    my $detone = $one->det();



    my $two = Math::MatrixReal->new_from_cols([    [$w,$i,$m,0],

                                                       [$in1,$in2,$in3,$in4],

                                                       [$c,$g,$y,$k],

                                                       [0,$d,$h,$z] ]);



    my $dettwo = $two->det();



    my $three = Math::MatrixReal->new_from_cols([    [$w,$i,$m,0],

                                                         [$f,$x,$j,$n],

                                                         [$in1,$in2,$in3,$in4],

                                                         [0,$d,$h,$z] ]);



    my $detthree = $three->det();



    my $four = Math::MatrixReal->new_from_cols([ [$w,$i,$m,0],

                                                     [$f,$x,$j,$n],

                                                     [$c,$g,$y,$k],

                                                     [$in1,$in2,$in3,$in4] ]);



    my $detfour = $four->det();



    my $pure1 = ($detone / $detA);

    my $pure2 = ($dettwo / $detA);

    my $pure3 = ($detthree / $detA);

    my $pure4 = ($detfour / $detA);

        

    return($pure1,$pure2,$pure3,$pure4);

}



sub aUT {

    my $max = $_[0];

    my $area = $_[1];

    if ($max <= $threshold) {

        $area = "UT";

    }

    return($area);

}

